package de.siedler_maps.mapinstaller;

import de.siedler_maps.mapinstaller.widgets.BitmapComponent;

import java.awt.*;
import java.awt.event.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import javax.swing.*;

import java.io.*;

/**
 * A class to download and install Settlers 6 maps from www.siedler-maps.de
 *
 * @author nevermind
 **/
public final class MapInstaller
extends JApplet
implements ActionListener, DownloadCallback
{
    /** Size of the file to download */
    private int filesize;
    /** Number of downloaded bytes (progress) */
    private int downloadProgress;
    
    /** Timer that updates the download progress */
    private SpeedMeter speed;
    /** Download thread */
    private Downloader download;
    
    /** language object. */
    private Language lang;
    
    /** Background image. */
    private final String background = "/de/siedler_maps/mapinstaller/data/background.png";
    /** Background color */
    private final Color backgroundColor = new Color(204, 204, 204);
    
    /** Message label */
    private JLabel messages;
    /** Download progress bar */
    private JProgressBar progressBar;
    /** Button */
    private JButton button;
    
    @Override
    public void init()
    {
        filesize = Integer.parseInt(getParameter("filesize"));
        
        // Init language object
        lang = new Language(this);
        
        // Add layout
        setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        
        // Set background
        add(new BitmapComponent(background));
        
        // Add components
        JPanel widgets = new JPanel();
        widgets.setPreferredSize(new Dimension(300, 200));
        widgets.setLayout(null);
        //widgets.setBackground(backgroundColor);
        add(widgets);
        
        // Message label
        messages = new JLabel();
        messages.setLocation(5, 5);
        messages.setSize(290, 30);
        messages.setHorizontalAlignment(JLabel.CENTER);
        widgets.add(messages);        
        
        // Progress bar
        progressBar = new JProgressBar();
        progressBar.setLocation(20, 40);
        progressBar.setSize(260, 20);
        progressBar.setMaximum(filesize);
        progressBar.setValue(0);
        widgets.add(progressBar);
        
        // Buttons
        button = new JButton(lang.get("mapinstaller/install"));
        button.setLocation(95, 70);
        button.setSize(100, 25);
        button.addActionListener(this);
        button.setActionCommand("install");
        widgets.add(button);
        
        // Show welcome message
        showMessage(lang.get("mapinstaller/message_click_to_start_download"));
    }
    
    /**
     * Shows a message.
     *
     * @param message The message to be shown.
     **/
    public void showMessage(final String message)
    {
        // This can be called out of another thread!
        EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                messages.setText(message);
            }
        });
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand() == "SpeedMeterCallback")
        {
            // Update speed meter
            speed.update(downloadProgress);
            // Calculate total download progress in percent
            float percent = ((float)downloadProgress / (float)filesize) * 100;

            String message = lang.get("mapinstaller/message_download_progress",
                    percent, speed.getSpeed() / 1024, downloadProgress / 1024, filesize / 1024);

            // Update UI
            progressBar.setValue(downloadProgress);
            showMessage(message);
        }
        else if (e.getActionCommand() == "install")
        {
            try
            {
                // URL, modulename and mapid are needed to create the link to the mapdownload
                String site_url     = getParameter("site_url");
                String modname      = getParameter("modname");
                int mapid           = Integer.parseInt(getParameter("mapid"));
                
                URL sourceURL       = new URL(String.format("%s/%s/download-%d.htm", site_url, modname, mapid));
               
                // Get the system's home directory
                String home = System.getProperty("user.home");
                // Build the install path for the map depending on the user's language
                String mapdir = lang.get("mapinstaller/mapdirectory");
                File installPath = new File(String.format("%s\\%s", home, mapdir));
                
                // Check if install directory exists
                if (!installPath.exists())
                {
                    showMessage(lang.get("mapinstaller/message_installdir_not_found"));
                    
                    // Hide button & progress bar
                    button.setVisible(false);
                    progressBar.setVisible(false);
                    
                    return;
                }

                // Mapname
                String mapname = getParameter("mapname");
                installPath = new File(installPath, mapname + ".zip");

                // Init progress update timer
                speed = new SpeedMeter(this);
                speed.start();

                button.setActionCommand("cancel");
                button.setText(lang.get("mapinstaller/cancel"));

                // Start download
                download = new Downloader(sourceURL, installPath, this);
                download.start();
            }
            catch (SecurityException ex)
            {
                showMessage(lang.get("mapinstaller/message_security_error"));
                
                // Hide button & progress bar
                button.setVisible(false);
                progressBar.setVisible(false);
                
                return;
            }
            catch (Exception ex)
            {
                return;
            }
        }
        else if (e.getActionCommand() == "cancel")
        {
            // Cancel download
            download.cancel();
        }
    }

    public void downloadUpdate(int bytesRead)
    {
        downloadProgress = bytesRead;
    }

    public void downloadFinished(boolean success)
    {
        // Stop progress timer
        speed.stop();
        // Hide button & progress bar
        button.setVisible(false);
        progressBar.setVisible(false);
        
        // Download failed
        if (!success)
        {
            // Show error message
            showMessage(lang.get("mapinstaller/message_download_failed"));
        }
        else
        {
            try
            {
                // Extract file(s)
                showMessage(lang.get("mapinstaller/message_extracting_file"));
            
                File archive = download.getDestination();
                
                ZipArchiveExtractor zip = new ZipArchiveExtractor();
                zip.extractArchive(archive, archive.getParentFile());

                // Delete the archive file
                archive.delete();
                
                showMessage(lang.get("mapinstaller/message_download_success"));
            }
            catch (Exception e)
            {
                // Unzip error - do something here?
            }
        }
    }
}
